unit snideicobut;

interface
uses windows, Controls, graphics,forms,Messages,Classes,themes;
type
  TButtonLayout = (blGlyphLeft, blGlyphRight, blGlyphTop, blGlyphBottom);
  TButtonState = (bsUp, bsDisabled, bsDown, bsExclusive);
  TButtonStyle = (bsAutoDetect, bsWin31, bsNew);
  TNumGlyphs = 1..4;
  ttextposition = (TP_Bottom,TP_Right,TP_Top,TP_left);
  TSnideIconButton = class(TCustomControl)

  private
    FState: TButtonState;
    ficon: ticon;
    fmousein,fflat,fdown: boolean;
    fGroupIndex: integer;
    fclick: TNotifyEvent;
    FAllowAllUp: boolean;
    fmodalresult: TModalResult;
    ftextposition: ttextposition;
    procedure EraseBack(var message: TWMEraseBkGnd); message WM_ERASEBKGND;
    procedure ColorChanged(var mes: tmessage); message CM_COLORCHANGED;
    procedure FontChanged(var mes: tmessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;


    function geticon: ticon;
    procedure seticon(const Value: ticon);
    procedure setflat(const Value: boolean);
    procedure PaintBack(bmp: tbitmap);
    procedure setdown( Value: boolean);
    procedure UpdateExclusive;
    procedure setftextposition(const Value: TTextPosition);
  public
    procedure Paint; override;

    constructor create(aowner: tcomponent); override;
    destructor destroy(); override;
    procedure Clicked;
  published
    property  Glyph: ticon read geticon write seticon;
    property font;
    property color;
    property caption;
    property enabled;
    property Flat: boolean read fflat write setflat;
    property Down: boolean read fdown write setdown;
    property GroupIndex: integer read fGroupIndex write fGroupIndex;
    property OnClick:TNotifyEvent read fclick write fclick;
    property AllowAllUp: boolean read FAllowAllUp write FAllowAllUp;
    property ModalResult: TModalResult read fmodalresult write fmodalresult;
    property TextPosition: TTextPosition read ftextposition write setftextposition;
  end;
procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SnideSoft',[TSnideIconButton]);
end;
procedure TSnideIconButton.UpdateExclusive;
var
  Msg: TMessage;
begin
  if (FGroupIndex <> 0) and (Parent <> nil) then
  begin
    Msg.Msg := CM_BUTTONPRESSED;
    Msg.WParam := FGroupIndex;
    Msg.LParam := Longint(Self);
    Msg.Result := 0;
    Parent.Broadcast(Msg);
  end;
end;
procedure TSnideIconButton.Clicked;
var
  Form: TCustomForm;
begin
  Form := GetParentForm(Self);
  if enabled   then
  begin
  if (Form <> nil) and (ModalResult <> mrNone) then
    Form.ModalResult := ModalResult
  else
  if assigned(fclick) then

    fclick(self);
  end;
end;

procedure TSnideIconButton.CMButtonPressed(var Message: TMessage);
var
  Sender: TSnideIconButton;
begin
  if Message.WParam = FGroupIndex then
  begin
    Sender := TSnideIconButton(Message.LParam);
    if Sender <> Self then
    begin
      if Sender.Down and FDown then
      begin
        FDown := False;
        FState := bsUp;
        {if (Action is TCustomAction) then
          TCustomAction(Action).Checked := False;}
        Invalidate;
      end;
      FAllowAllUp := Sender.AllowAllUp;
    end;
  end;
end;

procedure TSnideIconButton.CMEnabledChanged(var Message: TMessage);
begin
  //
end;

procedure TSnideIconButton.CMMouseEnter(var Message: TMessage);
var
  NeedRepaint: boolean;
begin

  fmousein := true;
  NeedRepaint := FFlat and not fmousein and Enabled and (DragMode <> dmAutomatic) and (GetCapture = 0);

  { Windows XP introduced hot states also for non-flat buttons. }
  if (NeedRepaint or ThemeServices.ThemesEnabled) and not (csDesigning in ComponentState) then
  begin
    
    if Enabled then
      Repaint;
  end;
end;

procedure TSnideIconButton.CMMouseLeave(var Message: TMessage);
var
  NeedRepaint: boolean;
begin
  fmousein := false;
  NeedRepaint := FFlat and not fmousein and Enabled and (DragMode <> dmAutomatic) and (GetCapture = 0);

  { Windows XP introduced hot states also for non-flat buttons. }
  if (NeedRepaint or ThemeServices.ThemesEnabled) and not (csDesigning in ComponentState) then
  begin
    
    if Enabled then
      Repaint;
  end;
end;

procedure TSnideIconButton.CMSysColorChange(var Message: TMessage);
begin
  paint;
end;

procedure TSnideIconButton.CMTextChanged(var Message: TMessage);
begin
  paint;
end;

procedure TSnideIconButton.ColorChanged(var mes: tmessage);
begin
  paint;
end;

constructor TSnideIconButton.create(aowner: tcomponent);
begin
  inherited;
  width:= 50;
  height:= 50;
  ficon := TIcon.Create;
end;

destructor TSnideIconButton.destroy;
begin
  ficon.Free;
  inherited;
end;

procedure TSnideIconButton.EraseBack(var message: TWMEraseBkGnd);
begin
  message.Result := 1;
end;

procedure TSnideIconButton.FontChanged(var mes: tmessage);
begin
  paint;
end;

function TSnideIconButton.geticon: ticon;
begin
  result := ficon;
end;

procedure TSnideIconButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    if not FDown then
    begin
      FState := bsDown;
      Invalidate;
    end;

  end;
end;

procedure TSnideIconButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if not fmousein then
  //  UpdateTracking;
end;

procedure TSnideIconButton.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
    
  if FGroupIndex = 0 then
    begin
      { Redraw face in-case mouse is captured }
      
      FState := bsUp;

      if DoClick and not (FState in [bsExclusive, bsDown]) then
        Invalidate;
    end
    else
      if DoClick then
      begin
        SetDown(not FDown);
        if FDown then Repaint;
      end
      else
      begin
        if FDown then FState := bsExclusive;
        Repaint;
      end;
    if DoClick then
      Clicked;
  paint;

end;

procedure TSnideIconButton.Paint;
var
  bmp : tbitmap;
  picleft,pictop,textlef,texttop,margin : integer;

begin
  if parent = nil then exit;
  bmp := TBitmap.Create;
  bmp.Width := width;
  bmp.Height := height;
  bmp.Canvas.Brush.Color := color;
  bmp.Canvas.FillRect(clientrect);

  PaintBack(bmp);

  bmp.Canvas.Font.Assign(font);
  margin := 0;
  if fstate= bsDown then
    margin := 1;


  case TextPosition of
  TP_left:
  begin
    picleft := width div 2 - (ficon.Width+bmp.Canvas.TextWidth(caption) +5) div 2 +bmp.Canvas.TextWidth(caption) +5 ;
    pictop := height div 2 - (ficon.height) div 2  ;
    textlef := width div 2 - (ficon.Width+bmp.Canvas.TextWidth(caption) +5) div 2;
    texttop := height div 2 - ( bmp.Canvas.TextHeight(Caption)) div 2 ;
  end;
  TP_right:
  begin
    picleft := width div 2 - (ficon.Width+bmp.Canvas.TextWidth(caption) +5) div 2 ;
    pictop := height div 2 - (ficon.height) div 2  ;
    textlef := width div 2 - (ficon.Width+bmp.Canvas.TextWidth(caption) +5) div 2 +ficon.Width ;
    texttop := height div 2 - ( bmp.Canvas.TextHeight(Caption)) div 2 ;
  end;
  TP_top:
  begin
    picleft := width div 2 - (ficon.Width) div 2 ;
    pictop := height div 2 - (ficon.height+bmp.Canvas.Textheight(caption) +2) div 2 + bmp.Canvas.TextHeight(Caption) +2 ;
    textlef := width div 2 - (bmp.Canvas.TextWidth(caption)) div 2  ;
    texttop := height div 2 - (ficon.height+ bmp.Canvas.TextHeight(Caption)) div 2 ;
  end;
  TP_bottom:
  begin
    picleft := width div 2 - (ficon.Width) div 2  ;
    pictop := height div 2 - (ficon.height+bmp.Canvas.Textheight(caption) +2) div 2  ;
    textlef := width div 2 - bmp.Canvas.TextWidth(caption) div 2;
    texttop := height div 2 - (ficon.height+ bmp.Canvas.TextHeight(Caption)+2) div 2 + ficon.Height +2;
  end;
  end;

  if not ficon.Empty then
  begin
    bmp.Canvas.Draw(picleft+margin,pictop+margin,ficon);
  end;
  bmp.Canvas.Brush.style := bsclear;
  bmp.Canvas.TextOut(textlef+margin,texttop+margin,caption);

  BitBlt(canvas.Handle,0,0,width,height,bmp.Canvas.Handle,0,0,srccopy);

  bmp.Free;
end;

procedure TSnideIconButton.PaintBack(bmp: tbitmap);
const
  DownStyles: array[Boolean] of Integer = (BDR_RAISEDINNER, BDR_SUNKENOUTER);
  FillStyles: array[Boolean] of Integer = (BF_MIDDLE, 0);
var
  PaintRect: TRect;
  DrawFlags: Integer;
  Offset: TPoint;
  Button: TThemedButton;
  ToolButton: TThemedToolBar;
  Details: TThemedElementDetails;
begin
  with bmp do
  begin
  if not Enabled then
  begin
    FState := bsDisabled;

  end
  else if FState = bsDisabled then
    if FDown and (GroupIndex <> 0) then
      FState := bsExclusive
    else
      FState := bsUp;
  Canvas.Font := Self.Font;

  if ThemeServices.ThemesEnabled then
  begin
    PerformEraseBackground(Self, Canvas.Handle);

    if not Enabled then
      Button := tbPushButtonDisabled
    else
      if FState in [bsDown, bsExclusive] then
        Button := tbPushButtonPressed
      else
        if fmousein then
          Button := tbPushButtonHot
        else
          Button := tbPushButtonNormal;

    ToolButton := ttbToolbarDontCare;
    if FFlat then
    begin
      case Button of
        tbPushButtonDisabled:
          Toolbutton := ttbButtonDisabled;
        tbPushButtonPressed:
          Toolbutton := ttbButtonPressed;
        tbPushButtonHot:
          Toolbutton := ttbButtonHot;
        tbPushButtonNormal:
          Toolbutton := ttbButtonNormal;
      end;
    end;

    PaintRect := ClientRect;
    if ToolButton = ttbToolbarDontCare then
    begin
      Details := ThemeServices.GetElementDetails(Button);
      ThemeServices.DrawElement(Canvas.Handle, Details, PaintRect);
      PaintRect := ThemeServices.ContentRect(Canvas.Handle, Details, PaintRect);
    end
    else
    begin
      Details := ThemeServices.GetElementDetails(ToolButton);
      ThemeServices.DrawElement(Canvas.Handle, Details, PaintRect);
      PaintRect := ThemeServices.ContentRect(Canvas.Handle, Details, PaintRect);
    end;

    if Button = tbPushButtonPressed then
    begin
      // A pressed speed button has a white text. This applies however only to flat buttons.
      if ToolButton <> ttbToolbarDontCare then
        Canvas.Font.Color := clHighlightText;
      Offset := Point(1, 0);
    end
    else
      Offset := Point(0, 0);
    {TButtonGlyph(FGlyph).Draw(Canvas, PaintRect, Offset, Caption, FLayout, FMargin, FSpacing, FState, Transparent,
      DrawTextBiDiModeFlags(0));    }
  end
  else
  begin
    PaintRect := Rect(0, 0, Width, Height);
    if not FFlat then
    begin
      DrawFlags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT;
      if FState in [bsDown, bsExclusive] then
        DrawFlags := DrawFlags or DFCS_PUSHED;
      DrawFrameControl(Canvas.Handle, PaintRect, DFC_BUTTON, DrawFlags);
    end
    else
    begin
      if (FState in [bsDown, bsExclusive]) or
        (fmousein and (FState <> bsDisabled)) or
        (csDesigning in ComponentState) then
        DrawEdge(Canvas.Handle, PaintRect, DownStyles[FState in [bsDown, bsExclusive]],
           BF_RECT)
      ;
      InflateRect(PaintRect, -1, -1);
    end;
    if FState in [bsDown, bsExclusive] then
    begin
      if (FState = bsExclusive) and (not FFlat or not fmousein) then
      begin
        Canvas.Brush.Bitmap := AllocPatternBitmap(clBtnFace, clBtnHighlight);
        Canvas.FillRect(PaintRect);
      end;
      Offset.X := 1;
      Offset.Y := 1;
    end
    else
    begin
      Offset.X := 0;
      Offset.Y := 0;
    end;
    {TButtonGlyph(FGlyph).Draw(Canvas, PaintRect, Offset, Caption, FLayout, FMargin,
      FSpacing, FState, Transparent, DrawTextBiDiModeFlags(0));   }
  end;
  end;
end;

procedure TSnideIconButton.setdown( Value: boolean);
begin
  if FGroupIndex = 0 then Value := False;

  if Value <> FDown then
  begin
    if FDown and (not FAllowAllUp) then Exit;
    FDown := Value;
    if Value then
    begin
      if FState = bsUp then Invalidate;
      FState := bsExclusive
    end
    else
    begin
      FState := bsUp;
      Repaint;
    end;
    if Value then UpdateExclusive;
  end;
end;

procedure TSnideIconButton.setflat(const Value: boolean);
begin
  fflat := Value;
  paint;
end;

procedure TSnideIconButton.seticon(const Value: ticon);
begin
  if value = nil then
  begin
    if not ficon.Empty then
      DestroyIcon(ficon.ReleaseHandle);
  end
  else
    ficon.Assign(Value);
  paint;
end;

procedure TSnideIconButton.setftextposition(const Value: TTextPosition);
begin
  ftextposition := Value;
  paint;
end;

end.
 