unit uInfCollectionComp;

{-----------------------------------------------------------------------------
Date: 01-SEP-2003
Author: Pawel Glowacki (Borland PSO EMEA)
Contact: pawel.glowacki@borland.com
Title: The Infinite Delphi Collection Component
Description:
This sample Delphi component demonstrates how-to create collections
of arbitrary depth.
Version: 1.0
-----------------------------------------------------------------------------}

interface

uses
  Classes;

type
  TInfCollectionItems = class; // forward declaration

  TInfCollectionItem = class(TCollectionItem)
  private
    FName: string;
    FItems: TInfCollectionItems;
    procedure SetItems(const Value: TInfCollectionItems);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    property Name: string read FName write FName;
    property Items: TInfCollectionItems read FItems write SetItems;
  end;

  TInfCollectionItems = class(TOwnedCollection)
  private
    function GetItem(Index: integer): TInfCollectionItem;
    procedure SetItem(Index: integer; const Value: TInfCollectionItem);
  public
    function Add: TInfCollectionItem;
    property Items[Index: integer]: TInfCollectionItem
      read GetItem write SetItem; default;
  end;

  TInfCollection = class(TComponent)
  private
    FItems: TInfCollectionItems;
    procedure SetItems(const Value: TInfCollectionItems);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Items: TInfCollectionItems read FItems write SetItems;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TInfCollection]);
end;

{ TInfCollection }

constructor TInfCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TInfCollectionItems.Create(self, TInfCollectionItem);
end;

destructor TInfCollection.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TInfCollection.SetItems(const Value: TInfCollectionItems);
begin
  FItems.Assign(Value);
end;

{ TInfCollectionItems }

function TInfCollectionItems.Add: TInfCollectionItem;
begin
  Result := TInfCollectionItem(inherited Add);
end;

function TInfCollectionItems.GetItem(Index: integer): TInfCollectionItem;
begin
  Result := TInfCollectionItem(inherited GetItem(Index));
end;

procedure TInfCollectionItems.SetItem(Index: integer;
  const Value: TInfCollectionItem);
begin
  inherited SetItem(Index, Value);
end;

{ TInfCollectionItem }

constructor TInfCollectionItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FItems := TInfCollectionItems.Create(self, TInfCollectionItem);
end;

destructor TInfCollectionItem.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TInfCollectionItem.SetItems(const Value: TInfCollectionItems);
begin
  FItems.Assign(Value);
end;

end.
